@@ -54,7 +54,7 @@ make install
 popd
 
 # Build freetype2
-pushd "$SRC/freetype2"
+pushd "$SRC/freetype"
 ./autogen.sh
 ./configure --prefix="$WORK" --disable-shared PKG_CONFIG_PATH="$WORK/lib/pkgconfig"
 make -j$(nproc)
@@ -94,7 +94,7 @@ popd
 
 # Build libjxl
 pushd "$SRC/libjxl"
-cmake . -DCMAKE_INSTALL_PREFIX=$WORK -DBUILD_TESTING=off -DJPEGXL_STATIC=true -DJPEGXL_FORCE_SYSTEM_LCMS2=true -DJPEGXL_ENABLE_EXAMPLES=false -DCMAKE_C_FLAGS="$CPPFLAGS $CFLAGS" -DCMAKE_CXX_FLAGS="$CPPFLAGS $CXXFLAGS"
+cmake . -DCMAKE_INSTALL_PREFIX=$WORK -DBUILD_TESTING=off -DBUILD_SHARED_LIBS=false -DJPEGXL_FORCE_SYSTEM_LCMS2=true -DJPEGXL_ENABLE_EXAMPLES=false -DJPEGXL_ENABLE_FUZZERS=false -DJPEGXL_ENABLE_TOOLS=false -DCMAKE_C_FLAGS="$CFLAGS" -DCMAKE_CXX_FLAGS="$CXXFLAGS"
 make -j$(nproc)
 make install
 popd
